module.exports = {
  env: {
    browser: true,
    es2021: true,
  },
  parser: "babel-eslint",
  extends: "airbnb-base",
  overrides: [{
    env: {
      node: true,
    },
    files: [
      ".eslintrc.{js,cjs,mjs}",
    ],
    parserOptions: {
      sourceType: "script",
    },
  }],
  "parserOptions": {
    "ecmaVersion": 6,
    "sourceType": "module",
    "ecmaFeatures": {
      "modules": true
    }
  },
  rules: {
    "callback-return": "warn",
    "class-methods-use-this": "off",
    "default-param-last": "off",
    "guard-for-in": "off",
    "import/extensions": "off",
    "import/named": "off",
    "import/no-relative-packages": "off",
    "import/no-unresolved": "off",
    "import/prefer-default-export": "off",
    "indent": ["error", 4, { "SwitchCase": 1 }],
    "max-classes-per-file": "off",
    "max-len": ["error", 170, 2, {
      ignoreUrls: true,
      ignoreComments: false,
      ignoreRegExpLiterals: true,
      ignoreStrings: true,
      ignoreTemplateLiterals: true,
    }],
    "max-lines": ["warn", 300],
    "no-await-in-loop": "warn",
    "no-bitwise": "off",
    "no-console": ["off", { allow: ["warn", "error", "info"] }], // warn for other
    "no-lone-blocks": "off",
    "no-loop-func": "off",
    "no-multiple-empty-lines": ["error", { max: 3, maxBOF: 0, maxEOF: 0 }],
    "no-negated-condition": 'off',
    "no-param-reassign": ["warn", {
      props: true,
      ignorePropertyModificationsFor: [
        "memo", // for reduce accumulators
        "acc", // for reduce accumulators
        "accumulator", // for reduce accumulators
        "e", // for e.returnvalue
        "ctx", // for Koa routing
        "context", // for Koa routing
        "req", // for Express requests
        "request", // for Express requests
        "res", // for Express responses
        "response", // for Express responses
        "$scope", // for Angular 1 scopes
        "staticContext", // for ReactRouter context
      ]
    }],
    "no-prototype-builtins": "off",
    "no-restricted-syntax": "off",
    "no-undef": "off",
    "no-underscore-dangle": "off",
    "no-unused-vars": "warn",
    "padding-line-between-statements": [
      "error",
      // First group: require an empty line between class declarations and the rest
      { blankLine: "always", prev: "class", next: "*" },
      { blankLine: "always", prev: "*", next: "class" },
      // Second group: require an empty line between function declarations and the rest
      { blankLine: "always", prev: "function", next: "*" },
      { blankLine: "always", prev: "*", next: "function" },
    ],
    "require-await": 'warn',
    "sort-keys": ['warn', 'asc', { caseSensitive: false, natural: true }],
  },
};
