<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->

- [RTLS Solution Services](#rtls-solution-services)
    - [Common](#common)
        - [Description](#description)
        - [Technologies](#technologies)
        - [Requirements](#requirements)
        - [Clone the project](#clone-the-project)
        - [Configuration](#configuration)
        - [Configure](#configure)
    - [Usage](#usage)
        - [Running the Service Locally](#running-the-service-locally)
        - [Project run tests locally](#project-run-tests-locally)
            - [Create tests](#create-tests)
            - [Unit tests](#unit-tests)
            - [Integration tests](#integration-tests)
    - [Package Release](#package-release)
    - [Conventions](#conventions)
        - [Code style](#code-style)
            - [Formatting rules](#formatting-rules)
            - [Naming rules](#naming-rules)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

# RTLS Solution Services

## Common

### Description

This GitHub repository is a comprehensive solution for building a back-end services that includes
a `Solutions Positions Manager` and `Solutions Subscriptions Manager`.
The repository is organized into three main components:

- `rtls-solutions-positions-manager`: Service that handles ZMQ messages from channel `$pos` and saves positions to the
  database.
- `rtls-solutions-subscriptions-manager`: Implements WebSocket API and re translate ZMQ messages from channels
  `$pos`, `#notify`, `#alarm`, `#param_p`, `#zone_event`, `#sensor`, `#ota` to WebSocket channels.

### Technologies

- `rtls-solutions-positions-manager`: Node.js, Express, MariaDB, Redis, ZMQ
- `rtls-solutions-subscriptions-manager`: Node.js, WebSocket, ZMQ

### Requirements

- Ubuntu 22.04
- NodeJs v20
- MariaDB v10.6.18-0
- Redis

### Clone the project

- Add public ssh key to [your account](https://github.com/settings/keys)

- Clone the repository

```shell
# via https
git clone https://github.com/rpplabs/rtls-solution-services.git

# OR via ssh key
git clone git@github.com:rpplabs/rtls-solution-services.git

cd rtls-solution-services

```

### Configuration

- setup projects
    - [rtls-api-gateway](https://github.com/rpplabs/rtls-api-gateway/blob/dev/README.md#project-local-setup)
    - [rtls-frontend](https://github.com/rpplabs/rtls-frontend/blob/dev/README.md#project-local-setup)
    - [rtls-historical-api-server](https://github.com/rpplabs/rtls-historical-api-server/blob/dev/README.md#project-local-setup)

- install `Redis`

```shell
sudo apt update
sudo apt install redis-server

```

- install positions manager node packages

```shell
cd ./apps/positions-manager
npm install

```

- install subscriptions manager node packages

```shell
cd ./apps/subscriptions-manager
npm install

```

### Configure

For packages configuration create `.env` files:

```shell
cp apps/positions-manager/.env.defaults apps/positions-manager/.env
cp apps/subscriptions-manager/.env.defaults apps/subscriptions-manager/.env

```

and update environment variables (e.g. database password)

## Usage

### Running the Service Locally

[positions manager](apps/positions-manager/README.md#running-the-service-locally)
[subscriptions manager](apps/subscriptions-manager/README.md#running-the-service-locally)

### Project run tests locally

[positions manager](apps/positions-manager/README.md#running-tests)
[subscriptions manager](apps/subscriptions-manager/README.md#running-tests)

#### Create tests

#### Unit tests

- unit tests should be places at the same level as the tested file
- unit test name should include the name of the file that is contains covered functionality and obey a
  pattern `{filename}.test.mjs`

#### Integration tests

- e2e tests should be places at the `e2e/integration` folder of each application
- e2e test name should include the name of the entity that is testing and obey a pattern `{filename}.test.mjs`

## Package Release

[positions manager](apps/positions-manager/README.md#package-release)
[subscriptions manager](apps/subscriptions-manager/README.md#package-release)

## Conventions

### Code style

#### Formatting rules

- please, follow the eslint rules described in project (with this purpose setting up your IDE with
  the [.eslintrc](.eslintrc));

#### Naming rules

- please name the variables and constants in a human-readable manner (ex. use `getUsersSql` instead of `sql1`);
- please use the `camelCase` style in name the variables and constants (ex. `getUsersSql` instead of `get_users_sql`);
- please use upper case naming for constants that are not changeable in project (ex. `MAX_ROWS_NUMBER`);
- please use `JavaScript` conventions in naming the classes, constructors(functions) and others entities (classes should
  be named wih the first capital latter);
