import * as zmq from 'zeromq';
// eslint-disable-next-line import/no-extraneous-dependencies
import { createLogger, LoggerTypes } from '@rtls-platform/logger/lib/index.mjs';
import { ZMQ_SUB_CHANNELS } from './lib/constants.mjs';
import * as ConfigContainer from './lib/config.cjs';
import { PositionManager, setLogger, controllers } from './lib/api/index.mjs';
import { getChannelWithMessage } from './lib/utils/index.mjs';

const sock = new zmq.Subscriber();

// eslint-disable-next-line import/no-mutable-exports
export let posManager;

export const main = async () => {
    try {
        sock.connect(ConfigContainer.config.zmqPubSubProxydUrl);
        // return NOTIFY subscription for platform stage 1
        sock.subscribe(ZMQ_SUB_CHANNELS.POS);

        const logger = createLogger({
            type: LoggerTypes.Winston,
            context: {
                logLevel: ConfigContainer.config.logLevel,
                serviceName: ConfigContainer.config.serviceName,
            },
        });
        logger.debug('ConfigContainer: ', ConfigContainer);

        setLogger(logger);

        process.on('unhandledRejection', (error) => {
            logger.error('unhandledRejection', error.stack);
        });

        process.on('uncaughtException', (error) => {
            logger.error('uncaughtException', error.stack);
        });

        posManager = new PositionManager(ConfigContainer.config);
        logger.info('Service start listen ZMQ SUB: ', ConfigContainer.config.zmqPubSubProxydUrl);

        for await (const [buffer] of sock) {
            try {
                const rawMessage = buffer.toString('utf8');
                const { channel, message } = getChannelWithMessage(rawMessage);
                const controller = controllers[channel];
                if (controller) {
                    await controller(message, posManager);
                } else {
                    logger.info('No controller for message: ', rawMessage);
                }
            } catch (e) {
                logger.error(e);
            }
        }
    } catch (err) {
        console.error(err);
        process.exit(1);
    }
};

if (process.argv.includes('main')) {
    main().then(() => {
        process.exit(0);
    }).catch((err) => {
        console.error(err);
        process.exit(1);
    });
}
