import assert from 'assert';
import * as zmq from 'zeromq';

import * as ConfigContainer from '../../lib/config.cjs';
import { main, posManager } from '../../app.mjs';
import { ZMQ_SUB_CHANNELS } from '../../lib/constants.mjs';

import { makePositionsIterator, delay } from './helpers.mjs';
import logger from '../../lib/api/logger.mjs';

const NUMBER_POS_TO_SEND = 100;

let sock;
let connection;

describe('Positions handling', () => {
    before(async () => {
        main();
        connection = await posManager.dbClient.pool.getConnection();

        sock = new zmq.Publisher();
        await sock.bind(ConfigContainer.config.zmqPubSubProxydUrl);

        if (ConfigContainer.config.solution_db.host === 'ubuntu22-mariadb') {
            const sql = `CREATE TABLE \`position_histories\` (
                \`id\` bigint(20) NOT NULL AUTO_INCREMENT,
                \`obj_uid\` char(22) NOT NULL,
                \`mac_address\` char(22) DEFAULT NULL,
                \`ts\` datetime(3) NOT NULL,
                \`x\` int(11) DEFAULT NULL,
                \`y\` int(11) DEFAULT NULL,
                \`z\` int(11) DEFAULT NULL,
                \`sublocation_uid\` char(22) NOT NULL,
                \`fom\` int(10) unsigned DEFAULT NULL,
                \`type\` varchar(45) DEFAULT NULL,
                PRIMARY KEY (\`id\`)
            ) ENGINE=InnoDB`;
            await connection.query(sql);
        }
    });
    after(async () => {
        await sock.unbind(ConfigContainer.config.zmqPubSubProxydUrl);
    });

    it('should pass positions handling', async () => {
        const posIt = makePositionsIterator({
            limit: NUMBER_POS_TO_SEND,
            mac: 'E4956EA43901',
            tk_uid: 'tko_test_3eal58CmyAGc2',
            sublocation_uid: '_UDqgghQR3eal58CmyAGc2',
        });
        await connection.query('truncate position_histories');
        let positionsSent = 0;
        await delay(500);

        for await (const pos of posIt) {
            // logger.info('send', `${ZMQ_SUB_CHANNELS.POS} ${JSON.stringify(pos)}`);
            await sock.send(`${ZMQ_SUB_CHANNELS.POS} ${JSON.stringify(pos)}`);
            positionsSent += 1;
        }
        // logger.info('++++++ TOTAL POS SENT: ', positionsSent);
        let posCount = 0;
        do {
            posCount = (await connection.query('select count(*) posCount from position_histories'))[0].posCount;
            // logger.info('result: ', posCount);
            await delay(300);
        } while (NUMBER_POS_TO_SEND > posCount);
        assert(NUMBER_POS_TO_SEND === posCount, 'Positions count mismatch');
        logger.info('unbind: ', ConfigContainer.config.zmqPubSubProxydUrl);
    }).timeout(50000);
});
