import { ZmqNotificator, UdpNotificator } from './types/index.mjs';

export const NotificatorTypes = {
    Zmq: 'ZMQ',
    Udp: 'UDP',
};

const cache = {};

export function createNotificator(
    options = {
        type: NotificatorTypes.Zmq,
        shared: true,
    },
) {
    let { type, shared } = options;
    const { config } = options;
    let notificator;

    type = type || NotificatorTypes.Zmq;
    shared = shared || true;

    if (shared) {
        notificator = cache[type];
        if (notificator) {
            return notificator;
        }
    }

    if (type === NotificatorTypes.Zmq) {
        notificator = new ZmqNotificator(config);
    } else if (type === NotificatorTypes.Udp) {
        notificator = new UdpNotificator(config);
    }

    if (shared) {
        cache[type] = notificator;
    }

    return notificator;
}
