import dgram from 'dgram';
import { CoreNotificator } from '../core/index.mjs';

export class UdpNotificator extends CoreNotificator {
    #transport = null;

    #port = null;

    #host = null;

    constructor(config) {
        super();

        const { port, host } = config;
        this.#transport = dgram.createSocket('udp4');
        this.#port = port;
        this.#host = host;

        if (!this.#transport) throw new Error('Transport not found');
    }

    formatMessage(channel, data) {
        return Buffer.from(`#${channel} ${JSON.stringify(data)}`);
    }

    async notify(channel, data) {
        return this.#sendMessage(this.formatMessage(channel, data));
    }

    #sendMessage = async (message) => {
        this.#transport.send(message, this.#port, this.#host, (err) => {
            if (err) {
                console.error(err);
            }
        });
    };
}
