import logger from '../../api/logger.mjs';
import ModelBase from '../ModelBase.mjs';

export class State extends ModelBase {
    /**
     * @typedef {Class} State
     * @property saveNodeProject
     * @property removeNodeProject
     * @property readNodeProject
     * @static init
     */

    /**
     * @param {Object} plist - payload
     * @param {string} plist.mac - hardware MAC address
     * @param {string} plist.projectUid - project unique identifier
     * @param {string} plist.companyUid - company unique identifier
     * @returns {Promise<Object>}
     */
    async saveNodeProject({ mac, projectUid, companyUid }) {
        if (!projectUid || !companyUid || !mac) {
            // eslint-disable-next-line prefer-rest-params
            logger.warn('Notify message has no projectUid/companyUid/mac/objUid: ', arguments);
            return undefined;
        }
        return this.repository.state.save({ mac, projectUid, companyUid });
    }

    /**
     * @param {Object} plist - payload
     * @param {string} plist.mac - hardware MAC address
     * @returns {Promise<Object>}
     */
    async removeNodeProject({ mac }) {
        if (!mac) {
            logger.warn('Notify message has no mac');
            return undefined;
        }
        return this.repository.state.remove(mac);
    }

    /**
     * @param {string} mac - hardware MAC address
     * @returns {Promise<{projectUid, companyUid}>}
     */
    async readNodeProject(mac) {
        return this.repository.state.read((mac || '').toUpperCase());
    }

    /**
     * @returns {Promise<undefined>}
     */
    static async init() {
        await State.repositoryInstance.state.clear();
        const nodes = await State.repositoryInstance.node.readAll();
        for (const node of nodes) {
            if (node.projectUid) {
                await State.repositoryInstance.state.save({ ...node, mac: (node.mac || '').toUpperCase() });
            }
        }
    }
}
