import assert from 'assert';
import { generateUid } from '../../utils/index.mjs';
import { State } from './State.mjs';
import ModelBase from '../ModelBase.mjs';

const projectUid = generateUid();
const companyUid = generateUid();
const repository = {
    state: {
        async read(mac) {
            return { mac, projectUid, companyUid };
        },
        // eslint-disable-next-line no-shadow
        async save({ mac, projectUid, companyUid }) {
            return { mac, projectUid, companyUid };
        },
        async remove(mac) {
            return mac;
        },
    },
};

describe('State', () => {
    before(() => {
        ModelBase.setRepository(repository);
    });

    it('should save node project', async () => {
        const mac = '00000000001a';
        const _projectUid = generateUid();
        const _companyUid = generateUid();
        const data = { mac, projectUid: _projectUid, companyUid: _companyUid };
        const result = await new State().saveNodeProject(data);
        assert.deepStrictEqual(result, data);
    });

    it('should remove node project', async () => {
        const mac = '00000000001a';
        const result = await new State().removeNodeProject({ mac });
        assert.strictEqual(result, mac);
    });

    it('should read node project', async () => {
        const mac = '00000000001a';
        const result = await new State().readNodeProject(mac);
        assert.deepStrictEqual(result, { mac: mac.toUpperCase(), projectUid, companyUid });
    });
});
