import * as fs from 'fs';
import logger from '../api/logger.mjs';

function readSecret(secretFilePath, defaultSecretFilePath) {
    function readFile(filePath) {
        try {
            return fs.readFileSync(filePath, 'utf8');
        } catch (e) {
            logger.error(`Fails to read secret file "${filePath}"`);
            logger.error(e);
            process.exit(-1);
            return undefined;
        }
    }

    if (fs.existsSync(defaultSecretFilePath)) {
        return readFile(defaultSecretFilePath);
    } if (fs.existsSync(secretFilePath)) {
        return readFile(secretFilePath);
    }
    return null;
}

/**
 * @function
 * @param {Object} authConfig - server configuration of the authentication part
 * @param {String} authConfig.secretFile - secret file path (file content is a jwt verification secret)
 * @param {String} authConfig.secretFileDefault - secret file default path (file content is a jwt verification secret) in case of wildcard absence
 * @param {String} authConfig.secret - predefined secret for case when no secret files were found
 * @return {undefined}
 */
export function setupAuthSecretSync(authConfig) {
    const secret = readSecret(authConfig.secretFile, authConfig.secretFileDefault);
    if (secret) {
        authConfig.secret = secret;
    }
}
