import { SubscriptionsManager, WsClient } from '../../models/index.mjs';
import { WS_CHANNELS, WS_ENTITY } from '../../system/index.mjs';
import UseCaseBase from '../UseCaseBase.mjs';

export class AddSubscriptions extends UseCaseBase {
    static validationRules = {
        channels: ['required', 'not_empty_list', {
            list_of: ['required', 'string', { one_of: [Object.values(WS_CHANNELS)] }],
        }],
        client: ['not_empty'],
        clientUid: ['string', 'required'],
        email: ['string', 'required'],
        entity: ['string', { eq: WS_ENTITY.SUBSCRIPTIONS }, 'required'],
        id: ['required'],
        jsonrpc: ['string', 'required'],
        project: ['string', 'required'],
    };

    async execute(params) {
        const wsClient = new WsClient(params.client, params.email, params.clientUid);
        const subscriptionsManager = new SubscriptionsManager();
        for (const channel of params.channels) {
            const topic = `${channel}:${params.project}`;
            subscriptionsManager.addSubscription(wsClient, topic, params.clientUid);
        }

        return JSON.stringify({
            id: params.id,
            jsonrpc: params.jsonrpc,
            result: params.channels,
        });
    }
}
