import { SubscriptionsManager } from '../../models/index.mjs';
import { WS_CHANNELS, WS_ENTITY } from '../../system/index.mjs';
import UseCaseBase from '../UseCaseBase.mjs';

export class DeleteSubscriptions extends UseCaseBase {
    static validationRules = {
        channels: ['required', 'not_empty_list', {
            list_of: ['required', 'string', { one_of: [Object.values(WS_CHANNELS)] }],
        }],
        clientUid: ['string', 'required'],
        email: ['string', 'required'],
        entity: ['string', { eq: WS_ENTITY.SUBSCRIPTIONS }, 'required'],
        id: ['required'],
        jsonrpc: ['string', 'required'],
        project: ['string', 'required'],
    };

    async execute(params) {
        const subscriptionsManager = new SubscriptionsManager();
        subscriptionsManager.deleteSubscriptionByParams(params.email, params.channels, params.project, params.clientUid);

        return JSON.stringify({
            id: params.id,
            jsonrpc: params.jsonrpc,
            result: params.channels,
        });
    }
}
