import { Notify, Dispatcher } from '../../models/index.mjs';
import UseCaseBase from '../UseCaseBase.mjs';

export class PositionHandle extends UseCaseBase {
    /**
     * @method
     * @param {Object} positionRaw  - raw position form JSON/BIN convertor
     * @param {String} positionRaw.ts  - position timestamp in ISO 8601 format
     * @param {String} positionRaw.tk_uid  - trackable object uid
     * @param {Object} positionRaw.data  - position properties
     * @param {Number} positionRaw.data.x  - the X coordinate of the position
     * @param {Number} positionRaw.data.y  - the Y coordinate of the position
     * @param {Number} positionRaw.data.z  - the Z coordinate of the position
     * @param {String} positionRaw.data.type  - positions type (UWB, HB)
     * @param {String} positionRaw.data.sl_uid  - sublocation numeric identifier
     * @param {String} positionRaw.data.mac  - hardware MAC address
     * @param {String} positionRaw.data.fom  - the figure of merit
     * @param {String} positionRaw.data.er  - 0D mode - distance to 0D anchor in millimeters
     * @param {String} positionRaw.data.slvr_mode  - the PE solver mode (NONE, PD, 0D, 1D, TDOA, TWOA, UNKNOWN)
     * @param {String} positionRaw.data.inactive  - the flag that indicated the tag mode: false - tag is moving, true - tag is not moving
     * @returns {Promise<null>}
     */
    async execute(positionRaw) {
        const { topic, message } = await Notify.getPositionMessage(positionRaw);
        await Dispatcher.send(topic, message);
        return null;
    }
}
