import UseCaseBase from '../UseCaseBase.mjs';
import { Dispatcher, Notify } from '../../models/index.mjs';

export class UserMsgHandle extends UseCaseBase {
    /**
     * @method
     * @param {Object} userMsgRawData - raw ota data
     * ...
     * @property {String} userMsgRawData.dev_id - Trackable unique id
     * @property {String} userMsgRawData.project_uid - Project unique id
     * @property {String} userMsgRawData.company_uid - Company unique id
     * @property {String} userMsgRawData.mac - Long mac address
     * @property {Number} userMsgRawData.ts - The seconds part of the timestamp
     * @property {Number} userMsgRawData.ms - The milliseconds part of the timestamp
     * @property {String} userMsgRawData.hex - max 128 binary bytes/(256 hex chars)
     * @see https://redpointpositioning.atlassian.net/wiki/spaces/EN/pages/1733034003/Solution+ZMQ+Message+Bus#%23user_msg
     * @todo extends require for CV project
     * @returns {Promise<null>}
     */
    async execute(userMsgRawData) {
        const { topic, message } = await Notify.getUserMsgMessage(userMsgRawData);
        await Dispatcher.send(topic, message);
        return null;
    }
}
