import RepoBase from '../RepoBase.mjs';
import { RepoError } from '../models/index.mjs';

export class NodeRepository extends RepoBase {
    /**
     * @typedef {Class} NodeRepository
     */

    /**
     * @param {Object} options - repository options
     * @param {Object} options.redis - redis connection
     * @param {Object} options.mariadb - mariadb connection
     * @param {string} options.prefix - redis storage prefix
     */
    constructor(options) {
        const { redis, mariadb, prefix } = options;

        super(mariadb, redis, prefix);
    }

    /**
     * @method
     * @return {Promise<Array<Object>>} returns all nodes
     */
    async readAll() {
        return this.mariadb.queryAsync('SELECT mac_address as mac, project_uid as projectUid, company_uid as companyUid FROM view_nodes')
            .catch((err) => {
                throw new RepoError(err);
            });
    }
}
